package goods

import (
	"encoding/json"
	"errors"
	"net/http"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"time"
)

// GetOrderList 获取商品订单
// 管理端 - 统计查询
func (c *GoodsController) GetOrderList() {
	caseID := c.GetString("caseid")
	if caseID == "" {
		c.ResponseError(errors.New("请选择有效案场"))
	}

	userType := c.GetString("usertype")
	status := c.GetString("status")
	orderStart := c.GetString("startdate")
	orderEnd := c.GetString("enddate")

	page, _ := c.GetInt("page")
	pagesize, _ := c.GetInt("pagesize")

	var orderDate []time.Time

	if orderStart != "" {
		t, err := time.Parse("20060102150405", orderStart)
		if err != nil {
			utils.LogError("接收时间参数失败: " + err.Error())
			c.ResponseError(errors.New("接收时间参数失败"))
		}

		orderDate = []time.Time{t}
	}

	if orderEnd != "" {
		t, err := time.Parse("20060102150405", orderEnd)
		if err != nil {
			utils.LogError("接收时间参数失败: " + err.Error())
			c.ResponseError(errors.New("接收时间参数失败"))
		}

		if orderDate == nil {
			orderDate = make([]time.Time, 0)
		}
		orderDate = append(orderDate, t)
	}

	list, total, err := c.serv.GetOrderList(caseID, userType, status, orderDate, page, pagesize)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(map[string]interface{}{
		"list":     list,
		"pagesize": pagesize,
		"pagenum":  total,
		"page":     page,
	})
}

// GetOnlineOrder 获取在线订单
// 管理端 - 有 websocket 操作
func (c *GoodsController) GetOnlineOrder() {
	caseID := c.GetString("caseid")
	if caseID == "" {
		c.ResponseError(errors.New("请选择有效案场"))
	}

	list, err := c.serv.GetOnLineOrders(caseID)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(map[string]interface{}{
		"list": list,
	})
}

// PostOrder 下单
// 微信端
func (c *GoodsController) PostOrder() {
	// 订单主信息
	info := c.GetString("info")
	if info == "" {
		c.ResponseError(errors.New("无有效订单信息"))
	}

	// 订单详情
	detail := c.GetString("detail")
	if detail == "" {
		c.ResponseError(errors.New("无有效订单详情信息"))
	}

	// 订单优惠券
	customercouponid := c.GetString("customercouponid")
	//
	var orderInfo model.TaGoodsOrders
	var orderDetail []model.TaGoodsOrdersDetail

	if err := json.Unmarshal([]byte(info), &orderInfo); err != nil {
		utils.LogError("下单转换JSON失败: " + err.Error())
		c.ResponseError(errors.New("下单数据格式不正确"))
	}

	if err := json.Unmarshal([]byte(detail), &orderDetail); err != nil {
		utils.LogError("下单详情转换JSON失败: " + err.Error())
		c.ResponseError(errors.New("详情数据格式不正确"))
	}

	// if err := json.Unmarshal([]byte(coupon), &orderCoupon); err != nil {
	// 	utils.LogError("下单优惠转换JSON失败: " + err.Error())
	// 	c.ResponseError(errors.New("优惠数据格式不正确"))
	// }
	cust := c.Context.Get("customer").(model.TaCustomer)

	if cust.Phone == "" {
		c.ResponseError(
			errors.New("用户未登录或绑定"),
			http.StatusNotAcceptable,
		)
	}

	if err := c.serv.Orders(&orderInfo, orderDetail, customercouponid); err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON("ok")
}

// GetOrdersByRecord 根据月记录导出订单信息
func (c *GoodsController) GetOrdersByRecord() {
	recordid := c.GetString(":recordid")

	record, err := c.caseserv.GetRecordByID(recordid)
	if err != nil {
		c.ResponseError(err)
	}
	if record == nil {
		c.ResponseError(errors.New("不存在对应的月记录！"))
	}

	orders, err := c.serv.GetOrdersByRecord(recordid)
	if err != nil {
		c.ResponseError(err)
	}

	excel, err := utils.NewTinyXLSXEngine()
	if err != nil {
		utils.LogError("初始化Excel服务失败: " + err.Error())
		c.ResponseError(errors.New("初始化Excel服务失败"))
	}

	excel.SetCell(excel.InsertRow(), []string{
		"归属案场",
		"商品分类",
		"商品名称",
		"商品规格",
		"下单人",
		"销售数量",
		"商品单价",
		"商品总价",
		"订单状态",
		"下单时间",
		"订单号",
	})

	var amount float64
	for _, order := range orders {
		for _, good := range order.Goods {
			row := excel.InsertRow()
			excel.SetCell(row, []string{
				order.CaseName,
				good.TypeName,
				good.GoodsName,
				good.SpecName,
				order.UserName,
				strconv.Itoa(good.Number),
				good.Price,
				strconv.Itoa(good.Number),
			})

			price, err := strconv.ParseFloat(good.Price, 64)
			if err != nil {
				c.ResponseError(errors.New("金额类型不正确！"))
			}
			// num, err := strconv.ParseFloat(strconv.Itoa(good.Number), 64)
			// if err != nil {
			// 	c.ResponseError(errors.New("金额类型不正确！"))
			// }
			charge := price * float64(good.Number)
			amount = amount + charge

			excel.SetCell(row, strconv.FormatFloat(charge, 'f', -1, 64))

			status := "未完成"
			if order.Status == models.STATUS_NORMAL {
				status = "已完成"
			}
			excel.SetCell(row, status)
			excel.SetCell(row, order.CreateDate.Format("2006-01-02 15:04:05"))
			excel.SetCell(row, order.OrdersNo)
		}
	}

	excel.SetCell(excel.InsertRow(), []string{
		"总规定次数",
		strconv.Itoa(record.Num),
		"实际使用总数",
		strconv.Itoa(record.ActrualNum),
		"总价",
		strconv.FormatFloat(amount, 'f', -1, 64),
	})

	c.SaveToExcel("订单月记录.xlsx", excel)
}

// GetCustomerOrders 获取用户订单
func (c *GoodsController) GetCustomerOrders() {
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")

	orders, err := c.serv.GetCustomerOrders(page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(orders)
}

// GetCaseOrdersNum 获取案场订单数量
func (c *GoodsController) GetCaseOrdersNum() {
	caseid := c.GetString(":caseid")
	num, err := c.serv.GetCaseOrdersNum(caseid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(num)
}

// FinishMake 完成制作
func (c *GoodsController) FinishMake() {
	orderid := c.GetString(":id")
	err := c.serv.FinishMake(orderid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功！")
}

// CancelMake 取消制作
func (c *GoodsController) CancelMake() {
	orderid := c.GetString(":id")
	err := c.serv.CancelMake(orderid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功！")
}
