package controllers

import (
	"bytes"
	"encoding/json"
	"errors"
	"io/ioutil"
	"net/http"
	"net/url"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/customer"
	"spaceofcheng/services/utils"
	"strings"

	"github.com/astaxie/beego"
)

// WechatInfo 微信接入
func (c *BaseController) WechatInfo() {
	// signature := c.GetString("signature")
	// timestamp := c.GetString("timestamp")
	// nonce := c.GetString("nonce")
	echostr := c.GetString("echostr")
	c.ResponseRaw([]byte(echostr))
}

// GetQrcodeURL 获取二维码信息
func (c *BaseController) GetQrcodeURL() {
	EquipmentID := c.GetString("EquipmentId")
	qrcodePrefix := beego.AppConfig.String("qrcodePrefix")
	org := c.Context.Get("org").(model.SysOrg)

	qrcode, err := utils.WxClientFor(org.OrgId).GetTempStrQRCode(qrcodePrefix + EquipmentID)
	if err != nil {
		utils.LogError("获取二维码失败！")
		c.ResponseError(err)
	}
	c.ResponseJSON(map[string]interface{}{
		"QrcodeUrl": qrcode,
	})
}

// WxReceive
func (c *BaseController) WxReceive() {
	org := c.Context.Get("org").(model.SysOrg)
	wechat := utils.WxClientFor(org.OrgId)
	r := c.Ctx.Request
	defer r.Body.Close()
	con, _ := ioutil.ReadAll(r.Body) //获取post的数据
	val, err := wechat.TransformMessage(string(con))
	if err != nil {
		beego.Error(err.Error())
		c.ResponseRaw([]byte(""))
	}

	if val["MsgType"] == "event" {
		qrcodePrefix := beego.AppConfig.String("qrcodePrefix")
		if (val["Event"] == "subscribe" && strings.Contains(val["EventKey"], "qrscene_"+qrcodePrefix)) || val["Event"] == "SCAN" {
			EquipmentID := strings.Split(val["EventKey"], qrcodePrefix)[1]
			messagetxt, err := c.scanDo(val, EquipmentID)
			if err != nil {
				beego.Error(err.Error())
				c.ResponseRaw([]byte(""))
			}
			c.ResponseRaw(messagetxt)
		}
	}
	c.ResponseRaw([]byte(""))
}

func (c *BaseController) scanDo(val map[string]string, EquipmentID string) ([]byte, error) {
	org := c.Context.Get("org").(model.SysOrg)
	wechat := utils.WxClientFor(org.OrgId)

	if val["FromUserName"] == "" {
		return nil, errors.New("openid为空")
	}

	user, err := wechat.GetUserDetail(val["FromUserName"])
	if err != nil {
		return nil, err
	}

	var customerserv = customer.NewCustomerServ(c.Context)

	// 判断用户是否存在
	getUser, err := customerserv.GetCustUserByOpenID(user["openid"].(string))
	if err != nil {
		return nil, err
	}
	name := user["nickname"].(string)
	name = UnicodeEmojiCode(name)
	user["nickname"] = name
	if getUser == nil || getUser.CustomerId != "" {
		// 新增用户
		_, err := customerserv.SaveNewCustomer(user, "", "", "", "")
		if err != nil {
			utils.LogError("用户保存失败：", err)
			return nil, err
		}
	}

	type WechatInfo struct {
		EquipmentId    string
		WXID           int
		WechatUserInfo map[string]interface{}
	}
	appid := wechat.GetAppID()
	user["wechataccount"] = user["openid"].(string)
	user["wechatofficial"] = appid
	var userinfo = WechatInfo{}
	userinfo.EquipmentId = EquipmentID
	userinfo.WechatUserInfo = user
	userinfo.WXID = 1

	apiPrefix := beego.AppConfig.String("postwechatInfo")

	b, err := json.Marshal(userinfo)
	if err != nil {
		return nil, err
	}

	body := bytes.NewBuffer([]byte(b))

	res, err := http.Post(apiPrefix, "application/json;charset=utf-8", body)
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	result, err := ioutil.ReadAll(res.Body)
	res.Body.Close()
	if err != nil {
		utils.LogError("发送消息失败：", err)
		return nil, err
	}
	var postResult struct {
		Status  bool
		Message string
	}
	if err = json.Unmarshal(result, &postResult); err != nil {
		utils.LogError("发送消息失败：", err)
		return nil, err
	}

	if !postResult.Status {
		beego.Error(postResult.Message)
		return nil, errors.New(postResult.Message)
	}

	messagetxt, err := wechat.ResponseMessageText(user["openid"].(string), beego.AppConfig.String("followText"))
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	return messagetxt, nil
}

// UnicodeEmojiCode 过滤表情
func UnicodeEmojiCode(s string) string {
	ret := ""
	rs := []rune(s)
	for i := 0; i < len(rs); i++ {
		if len(string(rs[i])) == 4 {
			// u := `[\u` + strconv.FormatInt(int64(rs[i]), 16) + `]`
			ret += ""
		} else {
			ret += string(rs[i])
		}
	}
	return ret
}

// GetWxJsSDKSignature 获取 微信 jssdk 签名
func (c *BaseController) GetWxJsSDKSignature() {
	org := c.Context.Get("org").(model.SysOrg)
	url, _ := url.QueryUnescape(c.GetString("url"))

	cli := utils.WxClientFor(org.OrgId)
	if cli == nil {
		c.ResponseError(errors.New("初始化微信服务失败"))
	}

	res := cli.GetJsTicketSignature(url)
	c.ResponseJSON(res)
}

// MiniLogin 获取小程序 OPENID
func (c *BaseController) MiniLogin() {
	org := c.Context.Get("org").(model.SysOrg)
	code := c.GetString("code")

	cli := utils.MiniClientFor(org.OrgId)
	if cli == nil {
		c.ResponseError(errors.New("初始化小程序服务失败"))
	}

	res, err := cli.GetOpenID(code)
	if err != nil {
		utils.LogError("获取用户信息失败: " + err.Error())
		c.ResponseError(errors.New("获取用户信息失败"))
	}

	c.ResponseJSON(res)
}

// DecodeMiniData 解密小程序信息
func (c *BaseController) DecodeMiniData() {
	org := c.Context.Get("org").(model.SysOrg)

	cli := utils.MiniClientFor(org.OrgId)
	if cli == nil {
		c.ResponseError(errors.New("初始化小程序服务失败"))
	}

	params, err := c.GetBodyData()
	if err != nil {
		utils.LogError("url数据解码失败: " + err.Error())
		c.ResponseError(errors.New("url数据解码失败"))
	}

	data, err := url.QueryUnescape(params["encrypted_data"].(string))
	key, err := url.QueryUnescape(params["session_key"].(string))
	iv, err := url.QueryUnescape(params["iv"].(string))

	res, err := cli.GetUserFromEncryptData(data, key, iv)
	if err != nil {
		utils.LogError("解密用户信息失败: " + err.Error())
		c.ResponseError(errors.New("解密用户信息失败"))
	}

	c.ResponseJSON(res)
}
