package coupon

import (
	"errors"
	"fmt"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/yl10/kit/guid"

	"github.com/go-xorm/xorm"
)

// CouponDAO 当前数据库操作对象
type CouponDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewCouponDAO New Inst
func NewCouponDAO(ctx *utils.Context) *CouponDAO {
	return &CouponDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// GetCouponByID 获取卡券信息
func (m *CouponDAO) GetCouponByID(id string) (*model.TaCoupon, error) {
	cp := model.TaCoupon{}

	if _, err := m.db.Where("coupon_id=?", id).Get(&cp); err != nil {
		return nil, err
	}

	return &cp, nil
}

// GetCouponInfoByID 获取优惠券详情
func (m *CouponDAO) GetCouponInfoByID(id string) (*CouponInfo, error) {
	cp := new(CouponInfo)

	query := `
		SELECT
			*
		FROM
			ta_coupon
		WHERE
			coupon_id = ?
	`

	if _, err := m.db.SQL(query, id).Get(cp); err != nil {
		return nil, err
	}

	// 规则
	var rules []model.TaCouponRule
	if err := m.db.Where("coupon_id=?", id).And("status=?", models.STATUS_NORMAL).Find(&rules); err != nil {
		return nil, err
	}

	// 图片
	var imgs []model.TaCouponImage
	if err := m.db.Where("coupon_id=?", id).Find(&imgs); err != nil {
		return nil, err
	}

	// 关联目标
	var targes []model.TaCouponTarget
	if err := m.db.Where("coupon_id=?", id).Find(&targes); err != nil {
		return nil, err
	}

	// 分享
	share, err := m.GetCouponShareByCouponID(id)
	if err != nil {
		return nil, err
	}

	cp.Targets = targes
	cp.Images = imgs
	cp.Rules = rules
	cp.Share = share

	channel, err := m.GetCouponChannel(id)
	if err != nil {
		return nil, err
	}
	if channel != nil {
		cp.ChannelId = channel.ChannelId
	}

	return cp, nil
}

// GetCouponInfoByCustomerCouponID 获取优惠券详情
func (m *CouponDAO) GetCouponInfoByCustomerCouponID(id string) (*CouponInfo, error) {
	cp := new(CouponInfo)

	query := `
		SELECT
			a.*
		FROM
			ta_coupon a 
			inner join ta_customer_coupon b 
			on a.coupon_id = b.coupon_id
		WHERE
			b.customer_coupon_id = ?
	`

	if _, err := m.db.SQL(query, id).Get(cp); err != nil {
		return nil, err
	}
	// 关联目标
	var targes []model.TaCouponTarget
	if err := m.db.Where("coupon_id=?", cp.CouponId).Find(&targes); err != nil {
		return nil, err
	}

	cp.Targets = targes

	return cp, nil
}

// GetCouponList 获取优惠券列表
func (m *CouponDAO) GetCouponList(filters []string, limit []int) ([]model.TaCoupon, int64, error) {
	var cps []model.TaCoupon

	filterString := ""
	if len(filters) > 0 {
		filterString = strings.Join(filters, " and ")
		filterString += " and "
	}

	sql := `select * from ta_coupon where %s status > ` + strconv.Itoa(models.STATUS_DEL) + ` order by create_date desc`

	total, err := utils.NewPageNaviEngine(m.ctx).GetPageList(&cps, fmt.Sprintf(sql, filterString), limit)
	if err != nil {
		return nil, 0, err
	}

	return cps, total, nil
}

// SaveCoupon 保存优惠券
func (m *CouponDAO) SaveCoupon(cp *model.TaCoupon) error {
	if cp.CouponId == "" {
		cp.CouponId = guid.NewGUIDString()
	}

	userRaw := m.ctx.Get("user")
	if userRaw != nil {
		user := userRaw.(model.SysUser)
		cp.CreateUser = user.UserId
	}

	org := m.ctx.Get("org").(model.SysOrg)
	cp.OrgId = org.OrgId

	cp.Status = models.STATUS_NORMAL
	cp.CreateDate = time.Now().Local()

	if _, err := m.db.Insert(cp); err != nil {
		return err
	}

	return nil
}

// UpdateCoupon 更新优惠券
func (m *CouponDAO) UpdateCoupon(cp *model.TaCoupon, cols []string) error {
	if cp.CouponId == "" {
		return errors.New("无优惠券信息")
	}

	if _, err := m.db.Cols(cols...).Where("coupon_id=?", cp.CouponId).Update(cp); err != nil {
		return err
	}

	return nil
}

// DeleteCoupon 删除优惠券
func (m *CouponDAO) DeleteCoupon(id string) error {
	if id == "" {
		return errors.New("无优惠券信息")
	}

	cp := model.TaCoupon{
		CouponId: id,
		Status:   models.STATUS_DEL,
	}

	if err := m.UpdateCoupon(&cp, []string{"status"}); err != nil {
		return err
	}

	return nil
}

// SaveCustomerCoupon 保存用户卡券
func (m *CouponDAO) SaveCustomerCoupon(cp *model.TaCustomerCoupon) error {
	if cp.CustomerCouponId == "" {
		cp.CustomerCouponId = guid.NewGUIDString()
	}

	cp.Status = models.STATUS_NORMAL

	if _, err := m.db.Insert(cp); err != nil {
		return err
	}

	return nil
}

// SaveCustomerCoupons 保存用户卡券
func (m *CouponDAO) SaveCustomerCoupons(cps []model.TaCustomerCoupon) error {
	for i := range cps {
		if cps[i].CustomerCouponId == "" {
			cps[i].CustomerCouponId = guid.NewGUIDString()
		}

		cps[i].Status = models.STATUS_NORMAL
	}

	if _, err := m.db.Insert(cps); err != nil {
		return err
	}

	return nil
}

// CheckCustCoupon 检查客户是否已经有了该卡券
// 如果用户有了卡券, 但是用过了，或者卡券过期了，仍然可以再次拥有
func (m *CouponDAO) CheckCustCoupon(custID, couponID string, startDate, endDate time.Time) ([]model.TaCustomerCoupon, error) {
	var cps []model.TaCustomerCoupon
	query := `
		SELECT
			*
		FROM
			ta_customer_coupon t
		WHERE
			t.coupon_id = ?
		AND t.customer_id = ?
		AND t.status = ?
		AND (
			(t.start_date BETWEEN ? AND ?)
			OR (t.end_date BETWEEN ? AND ?)
			OR (
				t.start_date <= ?
				AND t.end_date >= ?
			)
		)
	`
	if err := m.db.SQL(
		query,
		couponID,
		custID,
		models.STATUS_NORMAL,
		startDate,
		endDate,
		startDate,
		endDate,
		startDate,
		endDate,
	).Find(&cps); err != nil {
		return nil, err
	}

	return cps, nil
}

// SaveCouponRecoreds 保存优惠券记录
func (m *CouponDAO) SaveCouponRecoreds(recs []model.TaCouponGiveRecord) error {
	for i := range recs {
		if recs[i].RecId == "" {
			recs[i].RecId = guid.NewGUIDString()
		}
	}

	if _, err := m.db.Insert(&recs); err != nil {
		return err
	}

	return nil
}

// GetCouponBySendType 根据类型获取优惠券
func (m *CouponDAO) GetCouponBySendType(caseids, sendtype string) ([]model.TaCoupon, error) {
	var coupons []model.TaCoupon
	err := m.db.Where("case_id in ('"+strings.Replace(caseids, ",", "','", -1)+"')").And("send_type=?", sendtype).And("status=?", models.STATUS_NORMAL).Find(&coupons)
	return coupons, err
}

// GetCaseUsableCoupon 获取案场可用优惠券信息
func (m *CouponDAO) GetCaseUsableCoupon(caseid, userid string, page, pageSize int) ([]CaseCouponDetail, error) {
	var Coupons []CaseCouponDetail
	sql := `select * from ta_coupon where case_id=? and status=? and DATE_FORMAT(end_date, '%Y-%m-%d')>=DATE_FORMAT(NOW(), '%Y-%m-%d') and send_type=? order by create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql, caseid, models.STATUS_NORMAL, models.GIVE_TYPE_CASE).Find(&Coupons)
	for inx, coupon := range Coupons {
		if userid != "" {
			cCoupons, err := m.GetCustomerCouponByCouponAndUser(coupon.CouponId, userid)
			if err != nil {
				return nil, err
			}
			Coupons[inx].CustomerCoupon = cCoupons
		}
		share, err := m.GetCouponShareByCouponID(coupon.CouponId)
		if err != nil {
			return nil, err
		}
		Coupons[inx].Share = share
	}
	return Coupons, err
}

// GetCouponShareByCouponID 根据优惠券信息获取分享信息
func (m *CouponDAO) GetCouponShareByCouponID(couponid string) (*model.TaCouponShare, error) {
	var share = model.TaCouponShare{}
	_, err := m.db.Where("coupon_id=?", couponid).Get(&share)
	return &share, err
}

// GetCaseUsableCouponCount 获取案场可用优惠券数量
func (m *CouponDAO) GetCaseUsableCouponCount(caseid, userid string) (int, error) {
	var Coupons []CaseCouponDetail
	sql := `select * from ta_coupon where case_id=? and status=? and DATE_FORMAT(end_date, '%Y-%m-%d')>=DATE_FORMAT(NOW(), '%Y-%m-%d') and send_type=?`
	err := m.db.Sql(sql, caseid, models.STATUS_NORMAL, models.GIVE_TYPE_CASE).Find(&Coupons)

	return len(Coupons), err
}

// GetCustomerCouponByCouponAndUser 根据优惠券与用户获取领取的用户信息
func (m *CouponDAO) GetCustomerCouponByCouponAndUser(couponid, userid string) ([]model.TaCustomerCoupon, error) {
	var customerCoupons []model.TaCustomerCoupon
	err := m.db.Where("coupon_id=?", couponid).And("sales_id=?", userid).And("status>?", models.STATUS_DEL).Find(&customerCoupons)
	return customerCoupons, err
}

// GetCouponDetail 获取优惠券详情
func (m *CouponDAO) GetCouponDetail(couponid string) (*CaseCouponDetail, error) {
	var coupon = new(CaseCouponDetail)
	sql := `select * from ta_coupon where coupon_id=? and status>?`
	_, err := m.db.Sql(sql, couponid, models.STATUS_DEL).Get(coupon)
	if err != nil {
		return nil, err
	}
	if coupon == nil || coupon.CouponId == "" {
		return nil, errors.New("无优惠券信息")
	}
	cCoupons, err := m.GetCustomerCouponByCoupon(couponid)
	if err != nil {
		return nil, err
	}
	coupon.CustomerCoupon = cCoupons
	return coupon, nil
}

// GetCustomerCouponByCoupon 根据优惠券获取领取的用户信息
func (m *CouponDAO) GetCustomerCouponByCoupon(couponid string) ([]model.TaCustomerCoupon, error) {
	var customerCoupons []model.TaCustomerCoupon
	err := m.db.Where("coupon_id=?", couponid).And("status>?", models.STATUS_DEL).Find(&customerCoupons)
	return customerCoupons, err
}

// GetCustomerCouponByUser 根据用户获取领取的用户信息
func (m *CouponDAO) GetCustomerCouponByUser(caseid, userid string) ([]model.TaCustomerCoupon, error) {
	var customerCoupons []model.TaCustomerCoupon
	err := m.db.Where("case_id=?", caseid).And("sales_id=?", userid).And("status>?", models.STATUS_DEL).Find(&customerCoupons)
	return customerCoupons, err
}

// GetCouponWithCustomer 获取优惠券明细
func (m *CouponDAO) GetCouponWithCustomer(couponid string) (*CaseCouponDetail, error) {
	var coupon = new(CaseCouponDetail)
	sql := `select * from ta_coupon where coupin_id=?`
	_, err := m.db.Sql(sql, couponid).Get(&coupon)
	if err != nil {
		return nil, err
	}
	if coupon == nil || coupon.CouponId == "" {
		return nil, errors.New("无优惠券信息")
	}
	var customerCopons []model.TaCustomerCoupon
	sql = `select * from ta_customer_coupon where coupon_id = ? and status>?`
	err = m.db.Sql(sql, coupon, models.STATUS_DEL).Find(&customerCopons)
	if err != nil {
		return nil, err
	}
	coupon.CustomerCoupon = customerCopons
	return coupon, nil
}

// GetCouponByCustomer 获取我的优惠券信息
func (m *CouponDAO) GetCouponByCustomer(orgid, customerid string) ([]CustomerCoupon, error) {
	var coupons []CustomerCoupon
	sql := `select * from ta_customer_coupon where status>? and customer_id=? and org_id=?`
	err := m.db.Sql(sql, models.STATUS_DEL, customerid, orgid).Find(&coupons)
	// err := m.db.Where("status>?", models.STATUS_DEL).And("customer_id=?", customerid).And("org_id=?", orgid).Find(&coupons)
	for inx, coupon := range coupons {
		info, err := m.GetCouponInfoByID(coupon.CouponId)
		if err != nil {
			return nil, err
		}
		coupons[inx].Coupon = info
	}
	return coupons, err
}

// CustomerCouponWithShare 我的卡券关联分享信息
type CustomerCouponWithShare struct {
	model.TaCustomerCoupon `xorm:"extends"`
	Share                  *model.TaCouponShare
}

// GetCustomerCouponByID 获取我的优惠券详情
func (m *CouponDAO) GetCustomerCouponByID(id string) (*CustomerCouponWithShare, error) {
	if id == "" {
		return nil, errors.New("无优惠券信息")
	}
	var customerCoupon = new(CustomerCouponWithShare)
	sql := `select * from ta_customer_coupon where customer_coupon_id=?`
	_, err := m.db.Sql(sql, id).Get(customerCoupon)
	if err != nil {
		return nil, err
	}
	share, err := m.GetCouponShareByCouponID(id)
	if err != nil {
		return nil, err
	}
	customerCoupon.Share = share
	return customerCoupon, err
}

// GetCouponWithTarget 获取优惠券与关联信息
func (m *CouponDAO) GetCouponWithTarget(ids string) ([]CouponInfo, error) {
	var cp []CouponInfo

	query := `
		SELECT
			*
		FROM
			ta_coupon
		WHERE
			coupon_id in ('` + strings.Replace(ids, ",", "','", -1) + `')
	`

	if err := m.db.SQL(query).Find(&cp); err != nil {
		return nil, err
	}

	for i, c := range cp {
		// 关联目标
		var targes []model.TaCouponTarget
		if err := m.db.Where("coupon_id=?", c.CouponId).Find(&targes); err != nil {
			return nil, err
		}
		cp[i].Targets = targes
	}

	return cp, nil
}

// VerifyCustomerCoupon 核销客户优惠券
func (m *CouponDAO) VerifyCustomerCoupon(id string) error {
	var customerCoupon = model.TaCustomerCoupon{
		CustomerCouponId: id,
		UseDate:          time.Now(),
		Status:           models.STATUS_USED,
	}
	var cols = []string{
		"use_date",
		"status",
	}
	_, err := m.db.Cols(cols...).Where("customer_coupon_id=?", id).Update(customerCoupon)
	return err
}

// CheckBySerialCode 根据批次码判断
func (m *CouponDAO) CheckBySerialCode(couponid, code string) (bool, error) {
	var customerCoupons []model.TaCustomerCoupon
	err := m.db.Where("coupon_id=?", couponid).And("serial_code=?", code).Find(&customerCoupons)
	if err != nil {
		return false, err
	}
	if len(customerCoupons) == 0 {
		return true, nil
	}
	return false, nil
}

// CheckCustomerHaveCoupon 判断客户是否已领取
func (m *CouponDAO) CheckCustomerHaveCoupon(couponid, customerid string) (bool, error) {
	var customerCoupons []model.TaCustomerCoupon
	err := m.db.Where("coupon_id=?", couponid).And("customer_id=?", customerid).Find(&customerCoupons)
	if err != nil {
		return false, err
	}
	if len(customerCoupons) == 0 {
		return true, nil
	}
	return false, nil
}

// GetCustomerCouponByCustomerAndSaleCount 获取用户的领取记录
func (m *CouponDAO) GetCustomerCouponByCustomerAndSaleCount(customerid, saleid string) (int, error) {
	var customerCoupon []model.TaCustomerCoupon
	err := m.db.Where("customer_id=?", customerid).And("sales_id=?", saleid).Find(&customerCoupon)
	return len(customerCoupon), err
}

// GetCustomerCouponByCustomerAndSale 获取用户的领取记录
func (m *CouponDAO) GetCustomerCouponByCustomerAndSale(customerid, saleid string, page, pageSize int) ([]model.TaCustomerCoupon, error) {
	var customerCoupon []model.TaCustomerCoupon
	err := m.db.Where("customer_id=?", customerid).And("sales_id=?", saleid).Desc("status").Limit(pageSize, (page-1)*pageSize).Find(&customerCoupon)
	return customerCoupon, err
}

// GetNoCouponByCustomer 获取我的优惠券信息
func (m *CouponDAO) GetNoCouponByCustomer(orgid, customerid string) ([]model.TaCoupon, error) {
	var coupons []model.TaCoupon
	sql := `select * from ta_customer_coupon where status>? and use_date is null and customer_id=? and org_id=?`
	err := m.db.Sql(sql, models.STATUS_DEL, customerid, orgid).Find(&coupons)
	return coupons, err
}

// GetCouponChannel 获取优惠券对应渠道
func (m *CouponDAO) GetCouponChannel(couponid string) (*model.TaCardCouponChannel, error) {
	var channels []model.TaCardCouponChannel
	err := m.db.Where("card_coupon_type=?", models.COURSE_COUPON_COUPON).And("card_coupon_id=?", couponid).And("status=?", models.STATUS_NORMAL).Find(&channels)
	if err != nil {
		return nil, err
	}
	if len(channels) > 0 {
		return &channels[0], nil
	}
	return nil, nil
}

// DelCouponChannel 删除优惠券对应渠道
func (m *CouponDAO) DelCouponChannel(couponid string) error {
	sql := `delete from ta_card_coupon_channel where card_coupon_id=? and card_coupon_type=?`
	_, err := m.db.Exec(sql, couponid, models.COURSE_COUPON_COUPON)
	return err
}

// SaveCouponChannel 保存优惠券对应渠道
func (m *CouponDAO) SaveCouponChannel(channelid, couponid string) error {
	var couponChannel = model.TaCardCouponChannel{
		CardCouponChannelId: utils.GetGUID(),
		CardCouponId:        couponid,
		ChannelId:           channelid,
		CardCouponType:      models.COURSE_COUPON_COUPON,
		Status:              models.STATUS_NORMAL,
	}
	_, err := m.db.Insert(&couponChannel)
	return err
}
