package luckdraw

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"time"

	"github.com/go-xorm/xorm"
)

// LuckDrawDao 当前数据库操作对象
type LuckDrawDao struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewDAO 初始化DAO
func NewDAO(ctx *utils.Context) *LuckDrawDao {
	return &LuckDrawDao{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// LuckDraw 抽奖记录
type LuckDraw struct {
	model.TaLuckdraw `xorm:"extends"`
	Prizes           []model.TaLuckdrawPrize
}

// GetLuckDraw 获取抽奖信息
func (m *LuckDrawDao) GetLuckDraw(id string) (*LuckDraw, error) {
	if id == "" {
		return nil, errors.New("不存在抽奖信息！")
	}
	luckdraw, err := m.GetLuckDrawByID(id)
	if err != nil {
		utils.LogError("获取抽奖信息失败：", err)
		return nil, errors.New("获取抽奖信息失败")
	}
	prizes, err := m.GetPrizes(id)
	if err != nil {
		utils.LogError("获取奖品信息失败：", err)
		return nil, errors.New("获取奖品信息失败")
	}
	luckdraw.Prizes = prizes
	return luckdraw, nil
}

// LuckDraw 抽奖
func (m *LuckDrawDao) LuckDraw(id string, user *model.TaCustomer) (*model.TaLuckdrawPrize, *model.TaPrizeDetail, error) {
	luckdraw, err := m.GetLuckDrawByID(id)
	if err != nil {
		return nil, nil, errors.New("获取抽奖信息失败！")
	}
	if luckdraw.Status != models.STATUS_NORMAL {
		return nil, nil, errors.New("当前抽奖活动状态异常！")
	}
	if time.Now().Before(luckdraw.BeginDate) {
		return nil, nil, errors.New("活动还未开始！")
	}
	if time.Now().After(luckdraw.EndDate) {
		return nil, nil, errors.New("活动已结束！")
	}
	if user.Phone != "" {
		sysuser, err := m.GetUserByTel(user.Phone)
		if err != nil {
			return nil, nil, errors.New("获取信息失败！")
		}
		if sysuser != nil {
			return nil, nil, errors.New("内部人员不允许进行抽奖！")
		}
	}
	userluckdraws, err := m.GetUserRecordByLuckDraw(user.CustomerId, id)
	if err != nil {
		utils.LogError("获取用户抽奖信息失败:", err)
		return nil, nil, errors.New("获取用户抽奖信息失败！")
	}
	if len(userluckdraws) > 0 {
		return nil, nil, errors.New("您已参与过此次抽奖，不允许重复抽奖！")
	}
	prizes, err := m.GetPrizeStock(id)
	if err != nil {
		utils.LogError("获取奖项失败:", err)
		return nil, nil, errors.New("获取奖项失败！")
	}
	if len(prizes) == 0 {
		err = m.UpdateLuckDrawEndDate(luckdraw.Id)
		if err != nil {
			utils.LogError("操作失败，请刷新后重试:", err)
			return nil, nil, errors.New("操作失败，请刷新后重试！")
		}
		return nil, nil, errors.New("活动已结束！")
	}
	prize, err := m.GetWinning(prizes)
	if err != nil {
		utils.LogError("获取奖项失败:", err)
		return nil, nil, errors.New("获取奖项失败！")
	}

	// details, err := m.GetPrizeDetail(prize.Id)
	// if err != nil {
	// 	utils.LogError("获取奖项详情失败:", err)
	// 	return nil, nil, errors.New("获取奖项详情失败！")
	// }

	// 保存中奖记录
	var record = model.TaLuckdrawRecord{}
	record.Id = utils.GetGUID()
	record.OrgId = luckdraw.OrgId
	record.CaseId = luckdraw.CaseId
	record.PrizeId = prize.Id
	record.PrizeName = prize.PrizeName
	record.UserId = user.CustomerId
	record.UserName = user.CustomerName
	record.UserHeadImg = user.Headimgurl
	record.LuckdrawId = luckdraw.Id
	_, err = m.SaveRecord(record)
	// var detail = model.TaPrizeDetail{}
	// if len(details) > 0 {
	// 	// 更新中奖纪录明细
	// 	details[0].ReceiveId = newrecord.Id
	// 	err = m.UpdatePrizeDetail(details[0])
	// 	if err != nil {
	// 		utils.LogError("更新中奖纪录明细失败:", err)
	// 		return nil, nil, errors.New("操作失败！")
	// 	}
	// 	detail = details[0]
	// 	// 奖项自动核销，走卡券核销流程
	// 	err = m.PrizeWriteOff(newrecord.Id, user.CustomerId)
	// 	if err != nil {
	// 		utils.LogError("卡券核销失败:", err)
	// 		return nil, nil, errors.New("操作失败！")
	// 	}
	// }
	// 更新库存
	if prize.IsReality == 1 {
		prize.Remainder = prize.Remainder - 1
		err = m.UpdateStock(prize)
		if err != nil {
			return nil, nil, err
		}
		stockprizes, err := m.GetPrizeStock(id)
		if err != nil {
			utils.LogError("更新库存失败:", err)
			return nil, nil, errors.New("操作失败，请刷新后重试！")
		}
		if len(stockprizes) == 0 {
			// 更新活动截止时间
			err = m.UpdateLuckDrawEndDate(luckdraw.Id)
		}
	}

	return prize, nil, err
}

// PrizeWriteOff 奖品核销
func (m *LuckDrawDao) PrizeWriteOff(recordid, userid string) error {
	record, err := m.GetRecordByID(recordid)
	if err != nil {
		return err
	}
	var writeoff = model.TaLuckdrawWriteoff{}
	writeoff.LuckdrawId = record.LuckdrawId
	writeoff.CaseId = record.CaseId
	writeoff.OrgId = record.OrgId
	writeoff.RecordId = record.Id
	writeoff.WriteoffUser = userid
	_, err = m.SaveWirteOff(writeoff)
	if err != nil {
		return err
	}
	err = m.UpdateRecord(record)
	return err
}

// GetWinning 获取中奖信息
func (m *LuckDrawDao) GetWinning(prizes []model.TaLuckdrawPrize) (*model.TaLuckdrawPrize, error) {
	pList := make([]map[string]interface{}, 0)
	now := time.Now().Local()

	for _, prize := range prizes {
		// 有剩余的 并且 未过期的
		if prize.Remainder > 0 && now.Before(prize.VerificationEnd) {
			p := map[string]interface{}{
				"prize": prize,
				"prob":  prize.Probability,
			}

			pList = append(pList, p)
		}
	}

	if len(pList) == 0 {
		return nil, errors.New("所有奖品已抽完")
	}

	thePrize := utils.LuckyDraw(pList).(model.TaLuckdrawPrize)

	return &thePrize, nil
}

// GetLuckDrawByID 获取抽奖信息
func (m *LuckDrawDao) GetLuckDrawByID(id string) (*LuckDraw, error) {
	var luckdraws []LuckDraw
	sql := `select * from ta_luckdraw where id= ?`
	err := m.db.SQL(sql, id).Find(&luckdraws)
	if err != nil {
		return nil, err
	}
	if len(luckdraws) > 0 {
		return &luckdraws[0], nil
	}
	return nil, nil
}

// GetCaseAddressByLuckDarw 获取案场地址
func (m *LuckDrawDao) GetCaseAddressByLuckDarw(luckdraw string) (string, error) {
	sql := `
		SELECT
			s.case_address
		FROM
			ta_luckdraw t
		JOIN sys_case s ON t.org_id = s.org_id
		AND t.case_id = s.case_id
		AND t.id = ?
	`

	res, err := m.db.Query(sql, luckdraw)
	if err != nil {
		return "", err
	}

	if res == nil || len(res) == 0 {
		return "", nil
	}

	return string(res[0]["case_address"]), nil
}

// UpdateLuckDrawEndDate 更新活动结束时间
func (m *LuckDrawDao) UpdateLuckDrawEndDate(id string) error {
	var luckdraw = model.TaLuckdraw{
		Id:      id,
		EndDate: time.Now(),
	}
	cols := []string{
		"end_date",
	}
	_, err := m.db.Update(luckdraw, cols, "id=?", id)
	return err
}

// GetPrizes 获取奖品信息
func (m *LuckDrawDao) GetPrizes(luckdrawid string) ([]model.TaLuckdrawPrize, error) {
	var prizes []model.TaLuckdrawPrize
	err := m.db.Where("luckdraw_id=?", luckdrawid).And("status=?", models.STATUS_NORMAL).Find(&prizes)
	return prizes, err
}

// UpdateStock 修改库存
func (m *LuckDrawDao) UpdateStock(prize *model.TaLuckdrawPrize) error {
	cols := []string{
		"remainder",
	}
	_, err := m.db.Update(prize, cols, "id=?", prize.Id)
	return err
}

// GetPrizeStock 判断奖项库存
func (m *LuckDrawDao) GetPrizeStock(luckdrawid string) ([]model.TaLuckdrawPrize, error) {
	var prizes []model.TaLuckdrawPrize
	err := m.db.Where("luckdraw_id=?", luckdrawid).And("status=?", models.STATUS_NORMAL).And("(remainder>0 or is_reality=0)").Find(&prizes)
	return prizes, err
}

// GetPrizeByID 根据id获取奖品信息
func (m *LuckDrawDao) GetPrizeByID(id string) (model.TaLuckdrawPrize, error) {
	var prize = model.TaLuckdrawPrize{}
	_, err := m.db.Where("id=?", id).Get(&prize)
	return prize, err
}

// SaveRecord 保存抽奖记录
func (m *LuckDrawDao) SaveRecord(record model.TaLuckdrawRecord) (*model.TaLuckdrawRecord, error) {
	record.CreateDate = time.Now()
	record.Status = models.STATUS_READY
	record.Id = utils.GetGUID()
	_, err := m.db.Insert(&record)
	return &record, err
}

// GetUserByTel 根据电话获取用户信息
func (m *LuckDrawDao) GetUserByTel(tel string) (*model.SysUser, error) {
	var user = model.SysUser{}
	_, err := m.db.Where("phone=?", tel).Get(&user)
	return &user, err
}

// GetPrizeDetail 获取未领取的奖品详情
func (m *LuckDrawDao) GetPrizeDetail(prizeid string) ([]model.TaPrizeDetail, error) {
	var details []model.TaPrizeDetail
	err := m.db.Where("prize_id=?", prizeid).And("status=?", models.STATUS_READY).Find(&details)
	return details, err
}

// UpdatePrizeDetail 更新库存明细为已领取
func (m *LuckDrawDao) UpdatePrizeDetail(detail model.TaPrizeDetail) error {
	detail.Status = models.STATUS_NORMAL
	detail.ReceiveDate = time.Now()
	cols := []string{
		"status",
		"receive_id",
		"receive_date",
	}
	_, err := m.db.Update(detail, cols, "id=?", detail.Id)
	return err
}

// UpdateRecord 核销更新抽奖记录表
func (m *LuckDrawDao) UpdateRecord(record *model.TaLuckdrawRecord) error {
	record.Status = models.STATUS_NORMAL
	record.WriteoffDate = time.Now()
	cols := []string{
		"status",
		"writeoff_date",
	}
	_, err := m.db.Update(record, cols, "id=?", record.Id)
	return err
}

// SaveWirteOff 保存核销信息
func (m *LuckDrawDao) SaveWirteOff(writeoff model.TaLuckdrawWriteoff) (*model.TaLuckdrawWriteoff, error) {
	writeoff.Status = models.STATUS_NORMAL
	writeoff.Id = utils.GetGUID()
	writeoff.WriteoffDate = time.Now()
	_, err := m.db.Insert(&writeoff)
	return &writeoff, err
}

// GetUserRecordByLuckDraw 获取用户的抽奖信息
func (m *LuckDrawDao) GetUserRecordByLuckDraw(userid string, luckdrawid string) ([]model.TaLuckdrawRecord, error) {
	var records []model.TaLuckdrawRecord
	err := m.db.Where("user_id=?", userid).And("luckdraw_id=?", luckdrawid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&records)
	return records, err
}

// GetUserLuckDrawByLuckDraw 获取用户的抽奖信息
func (m *LuckDrawDao) GetUserLuckDrawByLuckDraw(userid string, luckdrawid string) (*model.TaLuckdrawRecord, error) {
	var records []model.TaLuckdrawRecord
	err := m.db.Where("user_id=?", userid).And("luckdraw_id=?", luckdrawid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&records)
	if err != nil {
		return nil, err
	}
	if len(records) > 0 {
		return &records[0], nil
	}
	return nil, nil
}

// GetDetailByRecord 获取detail
func (m *LuckDrawDao) GetDetailByRecord(recordid string) (*model.TaPrizeDetail, error) {
	var details []model.TaPrizeDetail
	err := m.db.Where("receive_id=?", recordid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&details)
	if err != nil {
		return nil, err
	}
	if len(details) > 0 {
		return &details[0], nil
	}
	return nil, nil
}

// UserRecord 用户抽奖信息
type UserRecord struct {
	model.TaLuckdrawRecord `xorm:"extends"`
	Url                    string
}

// GetUserLuckDraw 获取用户的抽奖信息
func (m *LuckDrawDao) GetUserLuckDraw(userid string) ([]UserRecord, error) {
	var records []UserRecord
	sql := `select a.*,b.url from ta_luckdraw_record a left join ta_prize_detail b on a.id=b.receive_id where a.user_id=? and a.status>` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql, userid).Find(&records)
	return records, err
}

// GetRecordByLuckDraw 抽奖记录
func (m *LuckDrawDao) GetRecordByLuckDraw(luckdrawid string) ([]model.TaLuckdrawRecord, error) {
	var records []model.TaLuckdrawRecord
	err := m.db.Where("luckdraw_id=?", luckdrawid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Desc("create_date").Find(&records)
	return records, err
}

// GetRecordByID 获取抽奖记录明细
func (m *LuckDrawDao) GetRecordByID(id string) (*model.TaLuckdrawRecord, error) {
	var record model.TaLuckdrawRecord
	_, err := m.db.Where("id=?", id).Get(&record)
	return &record, err
}

// SaveLuckDrawShareData 保存用户分享信息
func (m *LuckDrawDao) SaveLuckDrawShareData(dt *model.TaShareLuckyRecord) error {
	dt.CreateDate = time.Now().Local()
	dt.Status = models.STATUS_NORMAL
	dt.Id = utils.GetGUID()

	if _, err := m.db.Insert(dt); err != nil {
		return err
	}

	return nil
}

// GetLuckDrawShareData 获取用户分享内容
func (m *LuckDrawDao) GetLuckDrawShareData(from, to, luckID, caseID int) ([]model.TaShareLuckyRecord, error) {
	var dts []model.TaShareLuckyRecord

	if err := m.db.Where("from_customer_id=?", from).
		And("to_customer_id=?", to).
		And("luckydraw_id=?", luckID).
		And("case_id=?", caseID).
		And("status=?", models.STATUS_NORMAL).
		Find(&dts); err != nil {
		return nil, err
	}

	return dts, nil
}
