package luckdraw

import (
	"errors"
	"spaceofcheng/services/models/luckdraw"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
)

// LuckdrawServ 系统处理
type LuckdrawServ struct {
	ctx *utils.Context
	dao *luckdraw.LuckDrawDao
}

// NewLuckdrawServ 初始化
func NewLuckdrawServ(ctx *utils.Context) *LuckdrawServ {
	return &LuckdrawServ{
		ctx: ctx,
		dao: luckdraw.NewDAO(ctx),
	}
}

func (s *LuckdrawServ) GetLuckDraw(id string) (*luckdraw.LuckDraw, error) {
	luck, err := s.dao.GetLuckDraw(id)
	if err != nil {
		return nil, err
	}
	return luck, nil
}

func (s *LuckdrawServ) LuckDraw(id string, user *model.TaCustomer) (*model.TaLuckdrawPrize, *model.TaPrizeDetail, error) {
	prize, prizeDetail, err := s.dao.LuckDraw(id, user)
	if err != nil {
		return nil, nil, err
	}
	return prize, prizeDetail, nil
}

func (s *LuckdrawServ) PrizeWriteOff(recordid string, userid string) error {
	err := s.dao.PrizeWriteOff(recordid, userid)
	if err != nil {
		utils.LogError("奖品核销失败: " + err.Error())
		return errors.New("奖品核销失败")
	}
	return nil
}

func (s *LuckdrawServ) GetWinning(prizes []model.TaLuckdrawPrize) (*model.TaLuckdrawPrize, error) {
	prize, err := s.dao.GetWinning(prizes)
	if err != nil {
		return nil, err
	}
	return prize, nil
}

func (s *LuckdrawServ) GetLuckDrawByID(id string) (*luckdraw.LuckDraw, error) {
	luck, err := s.dao.GetLuckDrawByID(id)
	if err != nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		return nil, errors.New("获取抽奖信息失败")
	}
	return luck, nil
}

func (s *LuckdrawServ) GetCaseAddressByLuckDarw(luckdraw string) (string, error) {
	address, err := s.dao.GetCaseAddressByLuckDarw(luckdraw)
	if err != nil {
		utils.LogError("获取案场地址失败: " + err.Error())
		return "", errors.New("获取案场地址失败")
	}
	return address, nil
}

func (s *LuckdrawServ) UpdateLuckDrawEndDate(id string) error {
	err := s.dao.UpdateLuckDrawEndDate(id)
	if err != nil {
		utils.LogError("更新活动结束时间失败: " + err.Error())
		return errors.New("更新活动结束时间失败")
	}
	return nil
}

func (s *LuckdrawServ) GetPrizes(luckdrawid string) ([]model.TaLuckdrawPrize, error) {
	prize, err := s.dao.GetPrizes(luckdrawid)
	if err != nil {
		utils.LogError("获取奖品信息失败: " + err.Error())
		return nil, errors.New("获取奖品信息失败")
	}
	return prize, nil
}

func (s *LuckdrawServ) UpdateStock(prize *model.TaLuckdrawPrize) error {
	err := s.dao.UpdateStock(prize)
	if err != nil {
		utils.LogError("修改库存失败: " + err.Error())
		return errors.New("修改库存失败")
	}
	return nil
}

func (s *LuckdrawServ) GetPrizeStock(luckdrawid string) ([]model.TaLuckdrawPrize, error) {
	prize, err := s.dao.GetPrizeStock(luckdrawid)
	if err != nil {
		utils.LogError("判断奖项库存失败: " + err.Error())
		return nil, errors.New("判断奖项库存失败")
	}
	return prize, nil
}

func (s *LuckdrawServ) GetPrizeByID(id string) (model.TaLuckdrawPrize, error) {
	prize, err := s.dao.GetPrizeByID(id)
	if err != nil {
		utils.LogError("根据id获取奖品信息失败: " + err.Error())
		return prize, errors.New("根据id获取奖品信息失败")
	}
	return prize, nil
}

func (s *LuckdrawServ) SaveRecord(record model.TaLuckdrawRecord) (*model.TaLuckdrawRecord, error) {
	newRecord, err := s.dao.SaveRecord(record)
	if err != nil {
		utils.LogError("保存抽奖记录失败: " + err.Error())
		return nil, errors.New("保存抽奖记录失败")
	}
	return newRecord, nil

}

func (s *LuckdrawServ) GetPrizeDetail(prizeid string) ([]model.TaPrizeDetail, error) {
	detail, err := s.dao.GetPrizeDetail(prizeid)
	if err != nil {
		utils.LogError("获取未领取的奖品详情失败: " + err.Error())
		return nil, errors.New("获取未领取的奖品详情失败")
	}
	return detail, nil
}

func (s *LuckdrawServ) UpdatePrizeDetail(detail model.TaPrizeDetail) error {
	err := s.dao.UpdatePrizeDetail(detail)
	if err != nil {
		utils.LogError("更新库存明细为已领取失败: " + err.Error())
		return errors.New("更新库存明细为已领取失败")
	}
	return nil
}

func (s *LuckdrawServ) UpdateRecord(record *model.TaLuckdrawRecord) error {
	err := s.dao.UpdateRecord(record)
	if err != nil {
		utils.LogError("核销更新抽奖记录表失败: " + err.Error())
		return errors.New("核销更新抽奖记录表失败")
	}
	return nil
}

func (s *LuckdrawServ) SaveWirteOff(writeoff model.TaLuckdrawWriteoff) (*model.TaLuckdrawWriteoff, error) {
	newWriteoff, err := s.dao.SaveWirteOff(writeoff)
	if err != nil {
		utils.LogError("保存核销信息失败: " + err.Error())
		return nil, errors.New("保存核销信息失败")
	}
	return newWriteoff, nil
}

func (s *LuckdrawServ) GetUserRecordByLuckDraw(userid string, luckdrawid string) ([]model.TaLuckdrawRecord, error) {
	record, err := s.dao.GetUserRecordByLuckDraw(userid, luckdrawid)
	if err != nil {
		utils.LogError("获取用户的抽奖信息失败: " + err.Error())
		return nil, errors.New("获取用户的抽奖信息失败")
	}
	return record, nil
}

func (s *LuckdrawServ) GetUserLuckDrawByLuckDraw(userid string, luckdrawid string) (*model.TaLuckdrawRecord, error) {
	record, err := s.dao.GetUserLuckDrawByLuckDraw(userid, luckdrawid)
	if err != nil {
		utils.LogError("获取用户的抽奖信息失败: " + err.Error())
		return nil, errors.New("获取用户的抽奖信息失败")
	}
	return record, nil
}

func (s *LuckdrawServ) GetDetailByRecord(recordid string) (*model.TaPrizeDetail, error) {
	detail, err := s.dao.GetDetailByRecord(recordid)
	if err != nil {
		utils.LogError("获取detail失败: " + err.Error())
		return nil, errors.New("获取detail失败")
	}
	return detail, nil
}

func (s *LuckdrawServ) GetUserLuckDraw(userid string) ([]luckdraw.UserRecord, error) {
	record, err := s.dao.GetUserLuckDraw(userid)
	if err != nil {
		utils.LogError("获取用户的抽奖信息失败: " + err.Error())
		return nil, errors.New("获取用户的抽奖信息失败")
	}
	return record, nil

}

func (s *LuckdrawServ) GetRecordByLuckDraw(luckdrawid string) ([]model.TaLuckdrawRecord, error) {
	record, err := s.dao.GetRecordByLuckDraw(luckdrawid)
	if err != nil {
		utils.LogError("获取抽奖记录失败: " + err.Error())
		return nil, errors.New("获取抽奖记录失败")
	}
	return record, nil
}

func (s *LuckdrawServ) GetRecordByID(id string) (*model.TaLuckdrawRecord, error) {
	record, err := s.dao.GetRecordByID(id)
	if err != nil {
		utils.LogError("获取抽奖记录明细失败: " + err.Error())
		return nil, errors.New("获取抽奖记录明细失败")
	}
	return record, nil
}

func (s *LuckdrawServ) GetUserByOpenID(openid string) (*model.SysUser, error) {
	user, err := s.dao.GetUserByOpenID(openid)
	if err != nil {
		utils.LogError("根据 openid 获取用户人员信息失败: " + err.Error())
		return nil, errors.New("根据 openid 获取用户人员信息失败")
	}
	return user, nil
}

func (s *LuckdrawServ) GetUserByTel(tel string) (*model.SysUser, error) {
	user, err := s.dao.GetUserByTel(tel)
	if err != nil {
		utils.LogError("根据 tel 获取用户人员信息失败: " + err.Error())
		return nil, errors.New("根据 tel 获取用户人员信息失败")
	}
	return user, nil

}

func (s *LuckdrawServ) SaveLuckDrawShareData(dt *model.TaShareLuckyRecord) error {
	err := s.dao.SaveLuckDrawShareData(dt)
	if err != nil {
		utils.LogError("保存用户分享信息失败: " + err.Error())
		return errors.New("保存用户分享信息失败")
	}
	return nil

}
func (s *LuckdrawServ) GetLuckDrawShareData(luckID, from, to, caseID string) ([]model.TaShareLuckyRecord, error) {
	share, err := s.dao.GetLuckDrawShareData(luckID, from, to, caseID)
	if err != nil {
		utils.LogError("获取抽奖分享失败: " + err.Error())
		return nil, errors.New("获取抽奖分享失败")
	}
	return share, nil
}
