package statistics

import (
	"errors"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"time"
)

// StaDashboard 控制板统计
func (s *StatisticsServ) StaDashboard(caseIDs []string) ([][]interface{}, error) {
	// 人员及类型
	user := s.ctx.Get("user").(model.SysUser)
	userTypes, err := s.dao.GetUserTypes(user.UserId)
	if err != nil {
		utils.LogError("获取用户类型失败: " + err.Error())
		return nil, errors.New("获取用户类型失败")
	}

	tps := make([]string, 0)
	if userTypes != nil {
		for _, tp := range userTypes {
			tps = append(tps, tp.TypeId)
		}
	}

	// 统计配置
	settings, err := s.dao.GetDashboardSetting(user.UserId, tps)
	if err != nil {
		utils.LogError("获取用户 Dashboard 配置失败: " + err.Error())
		return nil, errors.New("获取用户 Dashboard 配置失败")
	}

	if settings == nil || len(settings) == 0 {
		return nil, nil
	}

	// 机构与案场
	org := s.ctx.Get("org").(model.SysOrg)
	// curcase := s.ctx.Get("currentCase").(model.SysUserCase)

	if caseIDs == nil || len(caseIDs) > 0 {
		allCases := s.ctx.Get("cases").([]model.SysUserCase)
		caseIDs = []string{}
		for _, cs := range allCases {
			caseIDs = append(caseIDs, cs.CaseId)
		}
	}

	// 日期
	now := time.Now().Local()
	y, m, d := now.Date()
	wd := int(now.Weekday())
	if wd == 0 {
		wd = 7
	}

	// 今天
	td := time.Date(y, m, d, 0, 0, 0, 0, time.Local)
	// 明天
	nd := td.Add(24 * time.Hour)
	// 后天
	ntd := nd.Add(24 * time.Hour)
	// 本周
	tw := td.Add(time.Duration((1-wd)*24) * time.Hour)
	// 下周
	nw := tw.Add(7 * 24 * time.Hour)
	// 本月
	tm := time.Date(y, m, 1, 0, 0, 0, 0, time.Local)
	// 下个月
	nm := time.Date(y, m+1, 1, 0, 0, 0, 0, time.Local)

	// 统计结果
	staDatas := make([][]interface{}, 0)
	res := make(map[string][]interface{})

	// 表配置顺序即为输出顺序
	for _, setting := range settings {
		data := map[string]interface{}{
			"type":   setting.CompType,
			"remark": setting.TypeName,
			"title":  setting.CompName,
		}

		switch setting.CompId {

		// -------
		// 运营

		// 统计会员总数
		case STA_TOTALCUSTOMER:
			dt, err := s.dao.GetCustomerNumber(org.OrgId, caseIDs, nil, nil)
			if err != nil {
				utils.LogError("统计会员总数失败: " + err.Error())
				return nil, errors.New("统计会员总数失败")
			}
			data["value"] = dt

		// 课程预约
		case STA_TOTALCOURSEORDER:
			dt, err := s.dao.GetCourseOrderNumber(org.OrgId, caseIDs, nil, nil)
			if err != nil {
				utils.LogError("统计课程预约失败: " + err.Error())
				return nil, errors.New("统计课程预约失败")
			}
			data["value"] = dt

		// 商品下单
		case STA_TOTALGOODS:
			dt, err := s.dao.GetOrderGoodsNumber(org.OrgId, caseIDs, nil, nil)
			if err != nil {
				utils.LogError("统计商品下单失败: " + err.Error())
				return nil, errors.New("统计商品下单失败")
			}
			data["value"] = dt

		// 今日数据
		case STA_TODAYDATA:
			list := make([]map[string]interface{}, 0)

			// 会员新增
			dt, err := s.dao.GetCustomerNumber(org.OrgId, caseIDs, &td, &nd)
			if err != nil {
				utils.LogError("统计会员新增失败: " + err.Error())
				return nil, errors.New("统计会员新增失败")
			}
			list = append(list, map[string]interface{}{
				"title": "推荐会员新增",
				"value": dt,
			})

			// 课程数
			dt, err = s.dao.GetCourseNumber(org.OrgId, caseIDs, &td, &nd)
			if err != nil {
				utils.LogError("统计课程数失败: " + err.Error())
				return nil, errors.New("统计课程数失败")
			}
			list = append(list, map[string]interface{}{
				"title": "课程数",
				"value": dt,
			})

			// 预约数
			dt, err = s.dao.GetCourseOrderNumber(org.OrgId, caseIDs, &td, &nd)
			if err != nil {
				utils.LogError("统计预约数失败: " + err.Error())
				return nil, errors.New("统计预约数失败")
			}
			list = append(list, map[string]interface{}{
				"title": "预约数",
				"value": dt,
			})

			// 到场人次
			dt, err = s.dao.GetArrivedCustomerNumber(org.OrgId, caseIDs, &td, &nd)
			if err != nil {
				utils.LogError("统计到场人次失败: " + err.Error())
				return nil, errors.New("统计到场人次失败")
			}
			list = append(list, map[string]interface{}{
				"title": "到场人次",
				"value": dt,
			})

			data["value"] = list

		// 明日课程
		case STA_TOMORROWCOURSEORDER:
			dt, err := s.dao.GetCourseOrderPie(org.OrgId, caseIDs, &nd, &ntd)
			if err != nil {
				utils.LogError("统计明日预约课程失败: " + err.Error())
				return nil, errors.New("统计明日预约课程失败")
			}
			data["value"] = dt

		// -------
		// 运营

		// 今日预约人数
		case STA_TODAYCOURSECUSTOMER:
			dt, err := s.dao.GetCourseCardCustomer(org.OrgId, caseIDs, &td, &nd)
			if err != nil {
				utils.LogError("统计今日预约人数失败: " + err.Error())
				return nil, errors.New("统计今日预约人数失败")
			}
			data["value"] = dt

		// 今日到场人数
		case STA_TODAY_ARRIVED:
			dt, err := s.dao.GetArrivedCustomerNumber(org.OrgId, caseIDs, &td, &nd)
			if err != nil {
				utils.LogError("统计到场人次失败: " + err.Error())
				return nil, errors.New("统计到场人次失败")
			}
			data["value"] = dt

		// 明日预约人数
		case STA_TOMORROWCOURSECUSTOMER:
			dt, err := s.dao.GetCourseCardCustomer(org.OrgId, caseIDs, &nd, &ntd)
			if err != nil {
				utils.LogError("统计今日预约人数失败: " + err.Error())
				return nil, errors.New("统计今日预约人数失败")
			}
			data["value"] = dt

		// 本月卡券总数
		case STA_MONTHCOUPONCARD:
			dt, err := s.dao.GetCouponCardNumber(org.OrgId, caseIDs, &tm, &nm)
			if err != nil {
				utils.LogError("统计卡券总数失败: " + err.Error())
				return nil, errors.New("统计卡券总数失败")
			}
			data["value"] = dt

		// 本月客户已使用
		case STA_MONTHUSEDCOUPONCARD:
			dt, err := s.dao.GetUsedCouponCardNumber(org.OrgId, caseIDs, &tm, &nm)
			if err != nil {
				utils.LogError("统计客户使用数失败: " + err.Error())
				return nil, errors.New("统计客户使用数失败")
			}
			data["value"] = dt

		// 本月已发放
		case STA_MONTHUNUSEDCOUPONCARD:
			dt, err := s.dao.GetCustomerCouponCardNumber(org.OrgId, caseIDs, &tm, &nm)
			if err != nil {
				utils.LogError("统计已发放卡券数失败: " + err.Error())
				return nil, errors.New("统计已发放卡券数失败")
			}
			data["value"] = dt

		// 本月饮品数
		case STA_MONTHGOODS:
			dt, err := s.dao.GetOrderGoodsNumber(org.OrgId, caseIDs, &tm, &nm)
			if err != nil {
				utils.LogError("统计商品下单失败: " + err.Error())
				return nil, errors.New("统计商品下单失败")
			}
			data["value"] = dt

		// 今天饮品数
		case STA_TODAYGOODS:
			dt, err := s.dao.GetOrderGoodsNumber(org.OrgId, caseIDs, &td, &nd)
			if err != nil {
				utils.LogError("统计商品下单失败: " + err.Error())
				return nil, errors.New("统计商品下单失败")
			}
			data["value"] = dt

		// 本月课程预约
		case STA_MONTHCOURSEORDER:
			dt, err := s.dao.GetCourseOrderPie(org.OrgId, caseIDs, &tm, &nm)
			if err != nil {
				utils.LogError("统计本月预约课程失败: " + err.Error())
				return nil, errors.New("统计本月预约课程失败")
			}
			data["value"] = dt

		// 本月课程参与率
		case STA_MONTHCOURSEPERCENT:
			dt1, err := s.dao.GetCourseOrderCustomer(org.OrgId, caseIDs, &tm, &nm)
			if err != nil {
				utils.LogError("统计本月预约人数失败: " + err.Error())
				return nil, errors.New("统计本月预约人数失败")
			}

			dt2, err := s.dao.GetUsedCourseOrderCustomer(org.OrgId, caseIDs, &tm, &nm)
			if err != nil {
				utils.LogError("统计本月核销人数失败: " + err.Error())
				return nil, errors.New("统计本月核销人数失败")
			}

			dt := float64(0.0)
			if dt2 != 0 {
				dt = float64(dt1/dt2) * 10
			}

			data["value"] = dt

		// 本周课程参与率
		case STA_WEEKCOURSEPERCENT:
			dt1, err := s.dao.GetCourseOrderCustomer(org.OrgId, caseIDs, &tw, &nw)
			if err != nil {
				utils.LogError("统计本周预约人数失败: " + err.Error())
				return nil, errors.New("统计本周预约人数失败")
			}

			dt2, err := s.dao.GetUsedCourseOrderCustomer(org.OrgId, caseIDs, &tw, &nw)
			if err != nil {
				utils.LogError("统计本周核销人数失败: " + err.Error())
				return nil, errors.New("统计本周核销人数失败")
			}

			dt := float64(0.0)
			if dt2 != 0 {
				dt = float64(dt1/dt2) * 10
			}

			data["value"] = dt

		// 本月饮品日下单
		case STA_GOODSDAILY:
			dt, err := s.dao.GetGoodsWithTypeDaily(org.OrgId, caseIDs)
			if err != nil {
				utils.LogError("统计本月饮品下单失败: " + err.Error())
				return nil, errors.New("统计本月饮品下单失败")
			}
			data["value"] = dt

		default:
			// 其余的不在本次中返回
			data["value"] = nil
		}

		typeData, exists := res[setting.CompType]
		if !exists {
			typeData = make([]interface{}, 0)
		}

		typeData = append(typeData, data)
		res[setting.CompType] = typeData

		// group 是按照序列来的, 约定必须符合自然数字序列
		if len(staDatas) <= setting.Group {
			staDatas = append(staDatas, typeData)
		} else {
			staDatas[setting.Group] = typeData
		}
	}

	return staDatas, nil
}

// GetStaFilterCharts 获取
func (s *StatisticsServ) GetStaFilterCharts(caseIDs []string, staType string) ([]interface{}, error) {
	// 人员及类型
	user := s.ctx.Get("user").(model.SysUser)
	userTypes, err := s.dao.GetUserTypes(user.UserId)
	if err != nil {
		utils.LogError("获取用户类型失败: " + err.Error())
		return nil, errors.New("获取用户类型失败")
	}

	tps := make([]string, 0)
	if userTypes != nil {
		for _, tp := range userTypes {
			tps = append(tps, tp.TypeId)
		}
	}

	// 统计配置
	settings, err := s.dao.GetDashboardSetting(user.UserId, tps)
	if err != nil {
		utils.LogError("获取用户 Dashboard 配置失败: " + err.Error())
		return nil, errors.New("获取用户 Dashboard 配置失败")
	}

	if settings == nil || len(settings) == 0 {
		return nil, nil
	}

	// 机构与案场
	org := s.ctx.Get("org").(model.SysOrg)
	// curcase := s.ctx.Get("currentCase").(model.SysUserCase)

	if caseIDs == nil || len(caseIDs) > 0 {
		allCases := s.ctx.Get("cases").([]model.SysUserCase)
		caseIDs = []string{}
		for _, cs := range allCases {
			caseIDs = append(caseIDs, cs.CaseId)
		}
	}

	res := make([]interface{}, 0)

	for _, setting := range settings {
		switch setting.CompId {
		case STA_FILTER_CUSTOMER:
			dt, err := s.dao.GetNewCustomerDaily(org.OrgId, caseIDs, staType)
			if err != nil {
				utils.LogError("统计日推荐会员失败: " + err.Error())
				return nil, errors.New("统计日推荐会员失败")
			}

			res = append(res, dt)

		case STA_FILTER_COURSE:
			dt, err := s.dao.GetCourseDaily(org.OrgId, caseIDs, staType)
			if err != nil {
				utils.LogError("统计日排课量失败: " + err.Error())
				return nil, errors.New("统计日排课量失败")
			}

			res = append(res, dt)

		case STA_FILTER_COURSE_ORDER:
			dt, err := s.dao.GetCourseOrderDaily(org.OrgId, caseIDs, staType)
			if err != nil {
				utils.LogError("统计日课程预约失败: " + err.Error())
				return nil, errors.New("统计日课程预约失败")
			}

			res = append(res, dt)

		case STA_FILTER_GOODS:
			dt, err := s.dao.GetGoodsDaily(org.OrgId, caseIDs, staType)
			if err != nil {
				utils.LogError("统计日饮品下单失败: " + err.Error())
				return nil, errors.New("统计日饮品下单失败")
			}

			res = append(res, dt)

		case STA_FILTER_ARRIVED:
			dt, err := s.dao.GetArrivedDaily(org.OrgId, caseIDs, staType)
			if err != nil {
				utils.LogError("统计日到场人次失败: " + err.Error())
				return nil, errors.New("统计日到场人次失败")
			}

			res = append(res, dt)
		}
	}

	return res, nil
}
